(*
  Delphi Ogg Vorbis encoder example.
  (c) copyright 2002 by Andrei Borovsky, borovsky@yandex.ru
  You may do with this file whatever you want, but
  remember that I'm not responsible for any consequences.
*)
unit CThrd;

interface

uses
  Classes, Ogg, Codec, VorbisEnc, wav_format;

const
  BytesIn = 8192;

type

  TOnDone = procedure of object;

  flBuf = array[0..0] of Single;
  PflBuf = ^flBuf;
  PPflBuf = ^PflBuf;

  TConvertThread = class(TThread)
  private
    { Private declarations }
  protected
  public
    InFile, OutFile : String;
    Progress, nBytes, fSize : Integer;
    SampleRate : Integer;
    Compression : Single;
    os : ogg_stream_state;
    og : ogg_page;
    op, nul : ogg_packet;
    vi : vorbis_info;
    vc : vorbis_comment;
    vd : vorbis_dsp_state;
    vb : vorbis_block;
    header, header_comm, header_code : ogg_packet;
    FIn, FOut : file of Byte;
    OnDone : TOnDone;
    procedure Execute; override;
    procedure CleanUp;
  end;

var
  ConvertThread : TConvertThread;
  Stop : Boolean = False;
  Idle : Boolean = True;
implementation

(*
 Most of the following routine is simply translated
 from the OggVorbis encoder example (which you can
 find in Vorbis SDK). See that example for additional
 comments.
*)

procedure TConvertThread.Execute;
var
  buf : array[0..BytesIn-1] of SmallInt;
  i, Bytes : Integer;
  buffer : PPFLOAT; (* see the note on the type
                     declaration in ogg.pas *)
  eos : Byte;
  Hdr : TWavHeader;
begin
while True do
begin
Idle := True;
if Terminated then Exit;
while OutFile<>'' do
begin
  Idle := False;
  Progress := 0;
  nBytes := 0;
  AssignFile(Fin, InFile);
  Reset(Fin);
  BlockRead(FIn, Hdr, WAVHeaderOffs);
  Self.SampleRate := Hdr.SampleRate;
  AssignFile(Fout, OutFile);
  ReWrite(Fout);
  eos := 0;
  fSize := FileSize(FIn);
  vorbis_info_init(vi);
  vorbis_encode_init_vbr(vi, 2, SampleRate, Compression);
  vorbis_comment_init(vc);
  vorbis_comment_add_tag(vc, 'ENCODER', 'Delphi_example_encoder');
  vorbis_analysis_init(vd, vi);
  vorbis_block_init(vd, vb);
  ogg_stream_init(os, Random(1000000000));
  vorbis_analysis_headerout(vd, vc, header, header_comm, header_code);
  ogg_stream_packetin(os, header);
  ogg_stream_packetin(os, header_comm);
  ogg_stream_packetin(os, header_code);
  while ogg_stream_flush(os, og) <> 0 do
  begin
    BlockWrite(FOut, og.header^, og.header_len);
    BlockWrite(FOut, og.body^, og.body_len);
  end;
  while eos = 0 do
  begin
    if not EOF(FIn) then
    begin
      BlockRead(Fin, buf, BytesIn, Bytes);
      Inc(nBytes, Bytes);
      Progress := Round((nBytes/fSize)*100);
      if Stop or Self.Terminated then
      begin
        CleanUp;
        Erase(Fout);
        Idle := True;
        Break;
      end;
      buffer := vorbis_analysis_buffer(vd, BytesIn shr 2);
      for i:=0 to (Bytes shr 2)-1 do
      begin
        (* C treats this variable as a 2-dimensional
         array. C knows the length of the rows
         of the array returned as a pointer and
         ObjectPascal doesn't. So here is the trick *)
        buffer^[i] := buf[i*2]/$8000;
        buffer^[i + vd.pcm_storage] := buf[i*2+1]/$8000;
      end;
      vorbis_analysis_wrote(vd, Bytes shr 2);
    end else vorbis_analysis_wrote(vd, 0);
    while vorbis_analysis_blockout(vd, vb) = 1 do
    begin
      vorbis_analysis(vb, nil);
      vorbis_bitrate_addblock(vb);
      while vorbis_bitrate_flushpacket(vd, op)= 1 do
      begin
        ogg_stream_packetin(os, op);
        while eos = 0 do
        begin
          if ogg_stream_pageout(os, og)=0 then Break;
          BlockWrite(FOut, og.header^, og.header_len);
          BlockWrite(FOut, og.body^, og.body_len);
          if ogg_page_eos(og) <> 0 then eos := 1;
        end;
      end;
    end;
  end;
  if not Idle then CleanUp;
end;
end;
end;

procedure TConvertThread.CleanUp;
begin
  ogg_stream_clear(os);
  vorbis_block_clear(vb);
  vorbis_dsp_clear(vd);
  vorbis_comment_clear(vc);
  vorbis_info_clear(vi);
  CloseFile(FIn);
  CloseFile(FOut);
  OutFile := '';
  Stop := False;
  OnDone;
end;

end.
