unit wav_format;

interface

const
  WAVHeaderOffs = 44;
  WAV_FORMAT_PCM = 1;
  WAV_FORMAT_ADPCM = 2;
  WAV_FORMAT_ALAW = 6;
  WAV_FORMAT_MULAW = 7;
  WAV_FORMAT_DVI_IMA_ADPCM = 17;
  WAV_FORMAT_MP3 = 85;


type
  TWAVHeader = record
    // RIFF file header
    RIFF: array [0..3] of Char;          // = 'RIFF'
    FileSize: Integer;                   // = FileSize - 8
    RIFFType: array [0..3] of Char;      // = 'WAVE'
    // Format chunk
    FmtChunkId: array [0..3] of Char;    // = 'fmt'
    FmtChunkSize: Integer;               // = 16
    FormatTag: Word;                     // One of WAV_FORMAT_XXX constants
    Channels: Word;                      // = 1 - mono = 2 - stereo
    SampleRate: Integer;
    BytesPerSecond: Integer;
    BlockAlign: Word;
    BitsPerSample: Word;                 // = 8, 16 or 32 Bits/sample
    // Data Chunk
    DataChunkId: array [0..3] of Char;   // = 'data'
    DataSize: Integer;   // Data size in bytes
  end;

  procedure FillHeaderPCM(var Header : TWAVHeader; Channels, Bits, SampleRate, RawSize : Integer);

implementation

procedure FillHeaderPCM(var Header : TWAVHeader; Channels, Bits, SampleRate, RawSize : Integer);
var
  text : array[0..4] of Char;
begin
  text := 'RIFF';
  Move(text[0], Header.RIFF[0], 4);
  Header.FileSize := RawSize + WAVHeaderOffs;
  text := 'WAVE';
  Move(text[0], Header.RIFFType[0], 4);
  text := 'fmt ';
  Move(text[0], Header.FmtChunkId[0], 4);
  Header.FmtChunkSize := 16;
  Header.FormatTag := WAV_FORMAT_PCM;
  Header.Channels := Channels;
  Header.SampleRate := SampleRate;
  Header.BitsPerSample := Bits;
  Header.BlockAlign := (Bits * Channels) shr 3;
  Header.BytesPerSecond := Samplerate * Header.BlockAlign;
  text := 'data';
  Move(text[0], Header.DataChunkId[0], 4);
  Header.DataSize := RawSize;
end;



end.
