(*
  Delphi/Kylix headers for OggVorbis software codec.
  Translated from codec.h header
  by Andrei Borovsky, borovsky@yandex.ru
  The original C/C++ headers and libraries (C) COPYRIGHT 1994-2001
  by the XIPHOPHORUS Company http://www.xiph.org/
*)

unit Codec;

interface

uses
  Ogg;

type

  PVORBIS_INFO = ^VORBIS_INFO;

  VORBIS_INFO = record
    version: Integer;
    channels: Integer;
    rate: LongInt;

    (* The below bitrate declarations are *hints*.
      Combinations of the three values carry the following implications:
      all three set to the same value:
      implies a fixed rate bitstream
      only nominal set:
      implies a VBR stream that averages the nominal bitrate. No hard
      upper/lower limit
      upper and or lower set:
      implies a VBR bitstream that obeys the bitrate limits. nominal
      may also be set to give a nominal rate.
      none set:
      the coder does not care to speculate. *)

    bitrate_upper: LongInt;
    bitrate_nominal: LongInt;
    bitrate_lower: LongInt;
    bitrate_window: LongInt;
    codec_setup: Pointer;
  end;

(* vorbis_dsp_state buffers the current vorbis audio
  analysis/synthesis state. The DSP state belongs to a specific
  logical bitstream *)

  PVORBIS_DSP_STATE = ^VORBIS_DSP_STATE;

  VORBIS_DSP_STATE = record
    analysisp: Integer;
    vi: PVORBIS_INFO;
    pcm: PPFLOAT;
    pcmret: PPFLOAT;
    pcm_storage: Integer;
    pcm_current: Integer;
    pcm_returned: Integer;
    preextrapolate: Integer;
    eofflag: Integer;
    lW: LongInt;
    W: LongInt;
    nW: LongInt;
    centerW: LongInt;
    granulepos: OGG_INT64_T;
    sequence: OGG_INT64_T;
    glue_bits: OGG_INT64_T;
    time_bits: OGG_INT64_T;
    floor_bits: OGG_INT64_T;
    res_bits: OGG_INT64_T;
    backend_state: Pointer;
  end;

  PALLOC_CHAIN = ^ALLOC_CHAIN;

  ALLOC_CHAIN = record
    ptr: Pointer;
    next: PALLOC_CHAIN;
  end;


  VORBIS_BLOCK = record
    // necessary stream state for linking to the framing abstraction
    pcm: PPFLOAT;
    // this is a pointer into local storage
    opb: OGGPACK_BUFFER;
    lW: LongInt;
    W: LongInt;
    nW: LongInt;
    pcmend: Integer;
    mode: Integer;
    eofflag: Integer;
    granulepos: OGG_INT64_T;
    sequence: OGG_INT64_T;
    vd: PVORBIS_DSP_STATE; // For read-only access of configuration
    (* local storage to avoid remallocing; it's up to the mapping to
    structure it *)
    localstore: Pointer;
    localtop: LongInt;
    localalloc: LongInt;
    totaluse: LongInt;
    reap: PALLOC_CHAIN;
    // bitmetrics for the frame
    glue_bits: LongInt;
    time_bits: LongInt;
    floor_bits: LongInt;
    res_bits: LongInt;
    internal: Pointer;
  end;

(* vorbis_block is a single block of data to be processed as part of
  the analysis/synthesis stream; it belongs to a specific logical
  bitstream, but is independant from other vorbis_blocks belonging to
  that logical bitstream. *)

(* vorbis_info contains all the setup information specific to the
  specific compression/decompression mode in progress (eg,
  psychoacoustic settings, channel setup, options, codebook
  etc). vorbis_info and substructures are in backends.h. *)

(* the comments are not part of vorbis_info so that vorbis_info can be
  static storage *)

  PVORBIS_COMMENT = ^VORBIS_COMMENT;

  VORBIS_COMMENT = record
  (* unlimited user comment fields. libvorbis writes 'libvorbis'
   whatever vendor is set to in encode *)
    user_comments: PPChar;
    comment_lengths: PInteger;
    comments: Integer;
    vendor: PChar;
  end;

(* libvorbis encodes in two abstraction layers; first we perform DSP
  and produce a packet (see docs/analysis.txt). The packet is then
  coded into a framed OggSquish bitstream by the second layer (see
  docs/framing.txt). Decode is the reverse process; we sync/frame
  the bitstream and extract individual packets, then decode the
  packet back into PCM audio. *)

(* The extra framing/packetizing is used in streaming formats, such as
  files. Over the net (such as with UDP), the framing and
  packetization aren't necessary as they're provided by the transport
  and the streaming layer is not used *)

// Vorbis PRIMITIVES: general

procedure vorbis_info_init(var vi: VORBIS_INFO); cdecl;

procedure vorbis_info_clear(var vi: VORBIS_INFO); cdecl;

function vorbis_info_blocksize(var vi: VORBIS_INFO;
                               zo: Integer): Integer; cdecl;

procedure vorbis_comment_init(var vc: VORBIS_COMMENT); cdecl;

procedure vorbis_comment_add(var vc: VORBIS_COMMENT;
                             comment: PChar); cdecl;

procedure vorbis_comment_add_tag(var vc: VORBIS_COMMENT;
                                 tag: PChar;
                                 contents: PChar); cdecl;

function vorbis_comment_query(var vc: VORBIS_COMMENT;
                              tag: PChar;
                              count: Integer): PChar; cdecl;

function vorbis_comment_query_count(var vc: VORBIS_COMMENT;
                                    tag: PChar): Integer; cdecl;

procedure vorbis_comment_clear(var vc: VORBIS_COMMENT) cdecl;

function vorbis_block_init(var v: VORBIS_DSP_STATE;
                           var vb: VORBIS_BLOCK): Integer; cdecl;

function vorbis_block_clear(var vb: VORBIS_BLOCK): Integer; cdecl;

procedure vorbis_dsp_clear(var v: VORBIS_DSP_STATE); cdecl;

// Vorbis PRIMITIVES: analysis/DSP layer

function vorbis_analysis_init(var v: VORBIS_DSP_STATE;
                              var vi: VORBIS_INFO): Integer; cdecl;

function vorbis_commentheader_out(var vc: VORBIS_COMMENT;
                                  var op: OGG_PACKET): Integer; cdecl;

function vorbis_analysis_headerout(var v: VORBIS_DSP_STATE;
                                   var vc: VORBIS_COMMENT;
                                   var op: OGG_PACKET;
                                   var op_comm: OGG_PACKET;
                                   var op_code: OGG_PACKET): Integer; cdecl;

function vorbis_analysis_buffer(var v: VORBIS_DSP_STATE;
                                vals: Integer): PPFLOAT; cdecl;

function vorbis_analysis_wrote(var v: VORBIS_DSP_STATE;
                               vals: Integer): Integer; cdecl;

function vorbis_analysis_blockout(var v: VORBIS_DSP_STATE;
                                  var vb: VORBIS_BLOCK): Integer; cdecl;

function vorbis_analysis(var vb: VORBIS_BLOCK;
                         op: POGG_PACKET): Integer; cdecl;

function vorbis_bitrate_addblock(var vb: VORBIS_BLOCK): Integer; cdecl;

function vorbis_bitrate_flushpacket(var vd: VORBIS_DSP_STATE;
                                    var op: OGG_PACKET): Integer; cdecl;

// Vorbis PRIMITIVES: synthesis layer

function vorbis_synthesis_headerin(var vi: VORBIS_INFO;
                                   var vc: VORBIS_COMMENT;
                                   var op: OGG_PACKET): Integer; cdecl;

function vorbis_synthesis_init(var v: VORBIS_DSP_STATE;
                               var vi: VORBIS_INFO): Integer; cdecl;

function vorbis_synthesis(var vb: VORBIS_BLOCK;
                          var op: OGG_PACKET): Integer; cdecl;

function vorbis_synthesis_blockin(var v: VORBIS_DSP_STATE;
                                  var vb: VORBIS_BLOCK): Integer; cdecl;

function vorbis_synthesis_pcmout(var v: VORBIS_DSP_STATE;
                                 var pcm: PPFLOAT): Integer; cdecl;

function vorbis_synthesis_read(var v: VORBIS_DSP_STATE;
                               samples: Integer): Integer; cdecl;

function vorbis_packet_blocksize(var vi: VORBIS_INFO;
                                 var op: OGG_PACKET): LongInt; cdecl;

// Vorbis ERRORS and return codes

const

  OV_FALSE = -1;
  OV_EOF = -2;
  OV_HOLE = -3;
  OV_EREAD = -128;
  OV_EFAULT = -129;
  OV_EIMPL = -130;
  OV_EINVAL = -131;
  OV_ENOTVORBIS = -132;
  OV_EBADHEADER = -133;
  OV_EVERSION = -134;
  OV_ENOTAUDIO = -135;
  OV_EBADPACKET = -136;
  OV_EBADLINK = -137;
  OV_ENOSEEK = -138;

implementation

const
  {$IFDEF LINUX}
  libname = 'libvorbis.so';
  {$ENDIF}
  {$IFDEF WIN32}
  libname = 'vorbis.dll';
  {$ENDIF}

  procedure vorbis_info_init; cdecl; external libname;
  procedure vorbis_info_clear; cdecl; external libname;
  function vorbis_info_blocksize; cdecl; external libname;
  procedure vorbis_comment_init; cdecl; external libname;
  procedure vorbis_comment_add; cdecl; external libname;
  procedure vorbis_comment_add_tag; cdecl; external libname;
  function vorbis_comment_query; cdecl; external libname;
  function vorbis_comment_query_count; cdecl; external libname;
  procedure vorbis_comment_clear; cdecl; external libname;
  function vorbis_block_init; cdecl; external libname;
  function vorbis_block_clear; cdecl; external libname;
  procedure vorbis_dsp_clear; cdecl; external libname;
  function vorbis_analysis_init; cdecl; external libname;
  function vorbis_commentheader_out; cdecl; external libname;
  function vorbis_analysis_headerout; cdecl; external libname;
  function vorbis_analysis_buffer; cdecl; external libname;
  function vorbis_analysis_wrote; cdecl; external libname;
  function vorbis_analysis_blockout; cdecl; external libname;
  function vorbis_analysis; cdecl; external libname;
  function vorbis_bitrate_addblock; cdecl; external libname;
  function vorbis_bitrate_flushpacket; cdecl; external libname;
  function vorbis_synthesis_headerin; cdecl; external libname;
  function vorbis_synthesis_init; cdecl; external libname;
  function vorbis_synthesis; cdecl; external libname;
  function vorbis_synthesis_blockin; cdecl; external libname;
  function vorbis_synthesis_pcmout; cdecl; external libname;
  function vorbis_synthesis_read; cdecl; external libname;
  function vorbis_packet_blocksize; cdecl; external libname;
end.
