(*
  Delphi/Kylix headers for OggVorbis software codec.
  Translated from vorbisfile.h header
  by Andrei Borovsky, borovsky@yandex.ru
  The original C/C++ headers and libraries (C) COPYRIGHT 1994-2001
  by the XIPHOPHORUS Company http://www.xiph.org/
*)

unit VorbisFile;

interface

uses
  {$IFDEF LINUX}
  Libc,
  {$ENDIF}
  Ogg,
  Codec;

(* The function prototypes for the callbacks are basically the same as for
  the stdio functions fread, fseek, fclose, ftell.
  The one difference is that the FILE* arguments have been replaced with
  a void* - this is to be used as a pointer to whatever internal data these
  functions might need. In the stdio case, it's just a FILE* cast to a void*

 If you use other functions, check the docs for these functions and return
 the right values. For seek_func(), you*MUST* return -1 if the stream is
 unseekable *)

type

  read_func_t = function(ptr : Pointer; size, nmemb : Integer; datasource : Pointer) : Integer; {$IFDEF WIN32} stdcall {$ELSE} cdecl {$ENDIF};
  seek_func_t = function(datasource : Pointer; offset : ogg_int64_t; whence : Integer) : Integer; {$IFDEF WIN32} stdcall {$ELSE} cdecl {$ENDIF};
  close_func_t = function(datasource : Pointer) : Integer; {$IFDEF WIN32} stdcall {$ELSE} cdecl {$ENDIF};
  tell_func_t = function(datasource : Pointer) : Integer; {$IFDEF WIN32} stdcall {$ELSE} cdecl {$ENDIF};

  OV_CALLBACKS = record
    read_func : read_func_t;
    seek_func : seek_func_t;
    close_func : close_func_t;
    tell_func : tell_func_t;
  end;

const

  NOTOPEN = 0;
  PARTOPEN = 1;
  OPENED = 2;
  STREAMSET = 3;
  INITSET = 4;

type
  OGGVORBIS_FILE = record
    datasource: Pointer; // Pointer to a FILE*, etc.
    seekable: Integer;
    offset: OGG_INT64_T;
    _end: OGG_INT64_T;
    oy: OGG_SYNC_STATE;
    (* If the FILE handle isn't seekable (eg, a pipe),
      only the current stream appears *)
    links: Integer;
    offsets: POGG_INT64_T;
    dataoffsets: POGG_INT64_T;
    serialnos: PLongInt;
    pcmlengths: POGG_INT64_T;
    vi: PVORBIS_INFO;
    vc: PVORBIS_COMMENT;
    // Decoding working state local storage
    pcm_offset: OGG_INT64_T;
    ready_state: Integer;
    current_serialno: LongInt;
    current_link: Integer;
    bittrack: Double;
    samptrack: Double;
    os: OGG_STREAM_STATE;
    (* take physical pages, weld into a logical
      stream of packets *)
    vd: VORBIS_DSP_STATE;
    // central working state for the packet->PCM decoder
    vb: VORBIS_BLOCK;
    // local working space for packet->PCM decode
    callbacks: OV_CALLBACKS;
  end;

function ov_clear(var vf: OGGVORBIS_FILE): Integer; cdecl;

(* Note by AB: The following routines rely on FILE
  record type and may be used only in Kylix, where
  functions working with such records are declared.
  In Delphi You can use only the *_callbacks versions
  of these routines. *)

{$IFDEF LINUX}

function ov_open(f: PIO_FILE;
                 var vf: OGGVORBIS_FILE;
                 initial: PChar;
                 ibytes: LongInt): Integer; cdecl;

function ov_test(f: PIO_FILE;
                 var vf: OGGVORBIS_FILE;
                 initial: PChar;
                 ibytes: LongInt): Integer; cdecl;

{$ENDIF}

function ov_open_callbacks(datasource: Pointer;
                           var vf: OGGVORBIS_FILE;
                           initial: PChar;
                           ibytes: LongInt;
                           callbacks: OV_CALLBACKS): Integer; cdecl;

function ov_test_callbacks(datasource: Pointer;
                           var vf: OGGVORBIS_FILE;
                           initial: PChar;
                           ibytes: LongInt;
                           callbacks: OV_CALLBACKS): Integer; cdecl;

function ov_test_open(var vf: OGGVORBIS_FILE): Integer; cdecl;

function ov_bitrate(var vf: OGGVORBIS_FILE;
                    i: Integer): LongInt cdecl;

function ov_bitrate_instant(var vf: OGGVORBIS_FILE): LongInt cdecl;

function ov_streams(var vf: OGGVORBIS_FILE): LongInt cdecl;

function ov_seekable(var vf: OGGVORBIS_FILE): LongInt cdecl;

function ov_serialnumber(var vf: OGGVORBIS_FILE;
                         i: Integer): LongInt cdecl;

function ov_raw_total(var vf: OGGVORBIS_FILE;
                      i: Integer): OGG_INT64_T cdecl;

function ov_pcm_total(var vf: OGGVORBIS_FILE;
                      i: Integer): OGG_INT64_T cdecl;

function ov_time_total(var vf: OGGVORBIS_FILE;
                       i: Integer): Double cdecl;

function ov_raw_seek(var vf: OGGVORBIS_FILE;
                     pos: LongInt): Integer cdecl;

function ov_pcm_seek(var vf: OGGVORBIS_FILE;
                     pos: OGG_INT64_T): Integer cdecl;

function ov_pcm_seek_page(var vf: OGGVORBIS_FILE;
                          pos: OGG_INT64_T): Integer cdecl;

function ov_time_seek(var vf: OGGVORBIS_FILE;
                      pos: Double): Integer cdecl;

function ov_time_seek_page(var vf: OGGVORBIS_FILE;
                           pos: Double): Integer cdecl;

function ov_raw_tell(var vf: OGGVORBIS_FILE): OGG_INT64_T cdecl;

function ov_pcm_tell(var vf: OGGVORBIS_FILE): OGG_INT64_T cdecl;

function ov_time_tell(var vf: OGGVORBIS_FILE): Double cdecl;

function ov_info(var vf: OGGVORBIS_FILE;
                 link : Integer): PVORBIS_INFO cdecl;

function ov_comment(var vf: OGGVORBIS_FILE;
                    link : Integer): PVORBIS_COMMENT cdecl;

function ov_read_float(var vf: OGGVORBIS_FILE;
                       var pcm_channels: PPFLOAT;
                       bitstream: PInteger): LongInt cdecl;

function ov_read(var vf: OGGVORBIS_FILE;
                 buffer: PChar;
                 length: Integer;
                 bigendianp: Integer;
                 word: Integer;
                 sgned: Integer;
                 bitstream: PInteger): LongInt cdecl;

implementation

const

  {$IFDEF LINUX}
  libname = 'libvorbisfile.so';
  {$ENDIF}
  {$IFDEF WIN32}
  libname = 'vorbisfile.dll';
  {$ENDIF}

{$IFDEF LINUX}
  function ov_open; cdecl; external libname;
  function ov_test; cdecl; external libname;
{$ENDIF}

  function ov_clear; cdecl; external libname;
  function ov_open_callbacks; cdecl; external libname;
  function ov_test_callbacks; cdecl; external libname;
  function ov_test_open; cdecl; external libname;
  function ov_bitrate; cdecl; external libname;
  function ov_bitrate_instant; cdecl; external libname;
  function ov_streams; cdecl; external libname;
  function ov_seekable; cdecl; external libname;
  function ov_serialnumber; cdecl; external libname;
  function ov_raw_total; cdecl; external libname;
  function ov_pcm_total; cdecl; external libname;
  function ov_time_total; cdecl; external libname;
  function ov_raw_seek; cdecl; external libname;
  function ov_pcm_seek; cdecl; external libname;
  function ov_pcm_seek_page; cdecl; external libname;
  function ov_time_seek; cdecl; external libname;
  function ov_time_seek_page; cdecl; external libname;
  function ov_raw_tell; cdecl; external libname;
  function ov_pcm_tell; cdecl; external libname;
  function ov_time_tell; cdecl; external libname;
  function ov_info; cdecl; external libname;
  function ov_comment; cdecl; external libname;
  function ov_read_float; cdecl; external libname;
  function ov_read; cdecl; external libname;
end.
