(*
  Delphi/Kylix headers for OggVorbis software codec.
  Translated from ogg.h and os_types.h headers
  by Andrei Borovsky, borovsky@yandex.ru
  The original C/C++ headers and libraries (C) COPYRIGHT 1994-2001
  by the XIPHOPHORUS Company http://www.xiph.org/
*)

unit ogg;

interface

type

  (* Type declarations from os_types.h header with
     some Delphi-specific types added *)

  OGG_INT64_T = int64;

  POGG_INT64_T = ^OGG_INT64_T;

  OGG_INT32_T = Integer;

  OGG_UINT32_T = LongWord;

  OGG_INT16_T = SmallInt;  // Word ?

  (* Note by AB: The following three type declarations,
   strange as they seem, are required to make C
   pointer/array stuff work in Delphi/Kylix. PPFLOAT
   is a conversion of float** C type, which is 
   treated as a 2-dimensional array. We treat it as
   a 1-dimensional since in Delphi/Kylix we cannot tell
   the size of an array pointed to by the pointer
   returned by a C function.
   See the encoder example for details. *)

  FLOAT_T = array[0..0] of Single;

  PFLOAT = ^FLOAT_T;

  PPFLOAT = ^PFLOAT;

  // Type declarations from ogg.h header

  POGGPACK_BUFFER = ^OGGPACK_BUFFER;

  OGGPACK_BUFFER = record
    endbyte: LongInt;
    endbit: Integer;
    buffer: PByte;
    ptr: PByte;
    storage: LongInt;
  end;

// ogg_page is used to encapsulate the data in one Ogg bitstream page

  OGG_PAGE = record
    header: PByte;
    header_len: LongInt;
    body: PByte;
    body_len: LongInt;
  end;

(* ogg_stream_state contains the current encode/decode state of a logical
 Ogg bitstream *)

  OGG_STREAM_STATE = record
    body_data: PByte;        // bytes from packet bodies
    body_storage: LongInt;   // storage elements allocated
    body_fill: LongInt;      // elements stored; fill mark
    body_returned: LongInt;  // elements of fill returned
    lacing_vals: PInteger;   // The values that will go to the segment table
    granule_vals: POGG_INT64_T; (* granulepos values for headers. Not compact
                                  this way, but it is simple coupled to the
                                  lacing fifo *)
    lacing_storage: LongInt;
    lacing_fill: LongInt;
    lacing_packet: LongInt;
    lacing_returned: LongInt;
    header: array[0..281] of Byte;  // working space for header encode
    header_fill: Integer;
    e_o_s: Integer;    (* set when we have buffered the last packet in the
                         logical bitstream *)
    b_o_s: Integer;    (* set after we've written the initial page
                         of a logical bitstream *)
    serialno: LongInt;
    pageno: LongInt;
    packetno: OGG_INT64_T;
    (* sequence number for decode; the framing
       knows where there's a hole in the data,
       but we need coupling so that the codec
      (which is in a seperate abstraction
      layer) also knows about the gap *)
    granulepos: OGG_INT64_T;
  end;

(* ogg_packet is used to encapsulate the data and metadata belonging
  to a single raw Ogg/Vorbis packet *)

  POGG_PACKET = ^OGG_PACKET;

  OGG_PACKET = record
    packet: PByte;
    bytes: LongInt;
    b_o_s: LongInt;
    e_o_s: LongInt;
    granulepos: OGG_INT64_T;
    packetno: OGG_INT64_T;
    (* sequence number for decode; the framing
       knows where there's a hole in the data,
       but we need coupling so that the codec
       (which is in a seperate abstraction
       layer) also knows about the gap  *)
  end;

  OGG_SYNC_STATE = record
    data: PByte;
    storage: Integer;
    fill: Integer;
    returned: Integer;
    unsynced: Integer;
    headerbytes: Integer;
    bodybytes: Integer;
  end;

// Ogg BITSTREAM PRIMITIVES: bitstream

procedure oggpack_writeinit(var b: OGGPACK_BUFFER); cdecl; 

procedure oggpack_reset(var b: OGGPACK_BUFFER); cdecl; 

procedure oggpack_writeclear(var b: OGGPACK_BUFFER); cdecl; 

procedure oggpack_readinit(var b: OGGPACK_BUFFER;
                           buf: PByte;
                           bytes: Integer); cdecl; 

procedure oggpack_write(var b: OGGPACK_BUFFER;
                        value: LongInt;
                        bits: Integer); cdecl; 

function oggpack_look(var b: OGGPACK_BUFFER;
                      bits: Integer): LongInt; cdecl; 

function oggpack_look_huff(var b: OGGPACK_BUFFER;
                           bits: Integer): LongInt; cdecl; 

function oggpack_look1(var b: OGGPACK_BUFFER): LongInt; cdecl; 

procedure oggpack_adv(var b: OGGPACK_BUFFER;
                      bits: Integer); cdecl; 

function oggpack_adv_huff(var b: OGGPACK_BUFFER;
                          bits: Integer): Integer; cdecl; 

procedure oggpack_adv1(var b: OGGPACK_BUFFER); cdecl; 

function oggpack_read(var b: OGGPACK_BUFFER;
                      bits: Integer): LongInt; cdecl; 

function oggpack_read1(var b: OGGPACK_BUFFER): LongInt; cdecl; 

function oggpack_bytes(var b: OGGPACK_BUFFER): LongInt; cdecl; 

function oggpack_bits(var b: OGGPACK_BUFFER): LongInt; cdecl; 

function oggpack_get_buffer(var b: OGGPACK_BUFFER): PByte; cdecl; 

// Ogg BITSTREAM PRIMITIVES: encoding

function ogg_stream_packetin(var os: OGG_STREAM_STATE;
                             var op: OGG_PACKET): Integer; cdecl; 

function ogg_stream_pageout(var os: OGG_STREAM_STATE;
                            var og: OGG_PAGE): Integer; cdecl; 

function ogg_stream_flush(var os: OGG_STREAM_STATE;
                          var og: OGG_PAGE): Integer; cdecl; 

// Ogg BITSTREAM PRIMITIVES: decoding

function ogg_sync_init(var oy: OGG_SYNC_STATE): Integer; cdecl; 

function ogg_sync_clear(var oy: OGG_SYNC_STATE): Integer; cdecl; 

function ogg_sync_reset(var oy: OGG_SYNC_STATE): Integer; cdecl; 

function ogg_sync_destroy(var oy: OGG_SYNC_STATE): Integer; cdecl; 

function ogg_sync_buffer(var oy: OGG_SYNC_STATE;
                         size: LongInt): PChar; cdecl; 

function ogg_sync_wrote(var oy: OGG_SYNC_STATE;
                        bytes: LongInt): Integer; cdecl; 

function ogg_sync_pageseek(var oy: OGG_SYNC_STATE;
                           var og: OGG_PAGE): LongInt; cdecl; 

function ogg_sync_pageout(var oy: OGG_SYNC_STATE;
                          var og: OGG_PAGE): Integer; cdecl; 

function ogg_stream_pagein(var os: OGG_STREAM_STATE;
                           var og: OGG_PAGE): Integer; cdecl; 

function ogg_stream_packetout(var os: OGG_STREAM_STATE;
                              var op: OGG_PACKET): Integer; cdecl; 

function ogg_stream_packetpeek(var os: OGG_STREAM_STATE;
                               var op: OGG_PACKET): Integer; cdecl; 

// Ogg BITSTREAM PRIMITIVES: general

function ogg_stream_init(var os: OGG_STREAM_STATE;
                         serialno: Integer): Integer; cdecl; 

function ogg_stream_clear(var os: OGG_STREAM_STATE): Integer; cdecl; 

function ogg_stream_reset(var os: OGG_STREAM_STATE): Integer; cdecl; 

function ogg_stream_destroy(var os: OGG_STREAM_STATE): Integer; cdecl; 

function ogg_stream_eos(var os: OGG_STREAM_STATE): Integer; cdecl; 

procedure ogg_page_checksum_set(var og: OGG_PAGE); cdecl;

function ogg_page_version(var og: OGG_PAGE): Integer; cdecl;

function ogg_page_continued(var og: OGG_PAGE): Integer; cdecl;

function ogg_page_bos(var og: OGG_PAGE): Integer; cdecl;

function ogg_page_eos(var og: OGG_PAGE): Integer; cdecl;

function ogg_page_granulepos(var og: OGG_PAGE): OGG_INT64_T; cdecl;

function ogg_page_serialno(var og: OGG_PAGE): Integer; cdecl;

function ogg_page_pageno(var og: OGG_PAGE): LongInt; cdecl;

function ogg_page_packets(var og: OGG_PAGE): Integer; cdecl;

procedure ogg_packet_clear(var op: OGG_PACKET); cdecl;

implementation

const

  {$IFDEF LINUX}
  libname = 'libogg.so';
  {$ENDIF}
  {$IFDEF WIN32}
  libname = 'ogg.dll';
  {$ENDIF}

  procedure oggpack_writeinit; cdecl; external libname;
  procedure oggpack_reset; cdecl; external libname;
  procedure oggpack_writeclear; cdecl; external libname;
  procedure oggpack_readinit; cdecl; external libname;
  procedure oggpack_write; cdecl; external libname;
  function oggpack_look; cdecl; external libname;
  function oggpack_look_huff; cdecl; external libname;
  function oggpack_look1; cdecl; external libname;
  procedure oggpack_adv; cdecl; external libname;
  function oggpack_adv_huff; cdecl; external libname;
  procedure oggpack_adv1; cdecl; external libname;
  function oggpack_read; cdecl; external libname;
  function oggpack_read1; cdecl; external libname;
  function oggpack_bytes; cdecl; external libname;
  function oggpack_bits; cdecl; external libname;
  function oggpack_get_buffer; cdecl; external libname;
  function ogg_stream_packetin; cdecl; external libname;
  function ogg_stream_pageout; cdecl; external libname;
  function ogg_stream_flush; cdecl; external libname;
  function ogg_sync_init; cdecl; external libname;
  function ogg_sync_clear; cdecl; external libname;
  function ogg_sync_reset; cdecl; external libname;
  function ogg_sync_destroy; cdecl; external libname;
  function ogg_sync_buffer; cdecl; external libname;
  function ogg_sync_wrote; cdecl; external libname;
  function ogg_sync_pageseek; cdecl; external libname;
  function ogg_sync_pageout; cdecl; external libname;
  function ogg_stream_pagein; cdecl; external libname;
  function ogg_stream_packetout; cdecl; external libname;
  function ogg_stream_packetpeek; cdecl; external libname;
  function ogg_stream_init; cdecl; external libname;
  function ogg_stream_clear; cdecl; external libname;
  function ogg_stream_reset; cdecl; external libname;
  function ogg_stream_destroy; cdecl; external libname;
  function ogg_stream_eos; cdecl; external libname;
  procedure ogg_page_checksum_set; cdecl; external libname;
  function ogg_page_version; cdecl; external libname;
  function ogg_page_continued; cdecl; external libname;
  function ogg_page_bos; cdecl; external libname;
  function ogg_page_eos; cdecl; external libname;
  function ogg_page_granulepos; cdecl; external libname;
  function ogg_page_serialno; cdecl; external libname;
  function ogg_page_pageno; cdecl; external libname;
  function ogg_page_packets; cdecl; external libname;
  procedure ogg_packet_clear; cdecl; external libname;
end.
